@@ -24,67 +24,85 @@ Desc.numeric <- function (x, main = NULL, highlow = TRUE, digits = NULL,
 
     probs= c(0, 0.05, 0.1, 0.25, 0.5, 0.75, 0.9, 0.95, 1)
 
-    ntot <- length(x)
+    ntot <- length(x) # total count
 
     x <- na.omit(x)
-    n <- length(x)
+    n <- length(x)    # now without NAs
 
-    NAs <- ntot - n
+    NAs <- ntot - n   # number of NAs
 
-    # the quantiles, totally analogue to the core of stats::quantile:
-    index <- 1 + (n - 1) * probs
-    lo <- floor(index)
-    hi <- ceiling(index)
-    x <- sort(x, partial = unique(c(lo, hi)))
-    qs <- x[lo]
-    i <- which(index > lo)
-    h <- (index - lo)[i]
-    qs[i] <- (1 - h) * qs[i] + h * x[hi[i]]
-    names(qs) <- c("min", ".05", ".10", ".25", "median", ".75", ".90", ".95", "max")
+    if(n > 0){
 
-    # ... here we go, all we need so far is in qs
+      # the quantiles, totally analogue to the core of stats::quantile:
+      index <- 1 + (n - 1) * probs
+      lo <- floor(index)
+      hi <- ceiling(index)
+      x <- sort(x, partial = unique(c(lo, hi)))
+      qs <- x[lo]
+      i <- which(index > lo)
+      h <- (index - lo)[i]
+      qs[i] <- (1 - h) * qs[i] + h * x[hi[i]]
+      names(qs) <- c("min", ".05", ".10", ".25", "median", ".75", ".90", ".95", "max")
 
-    # proceed with the parameteric stuff
-    meanx <- mean.default(x)      # somewhat faster than mean
+      # ... here we go, all we need so far is in qs
 
-    # we send the SORTED vector to the C++ function to calc the power sum(s)
-    psum <- .Call("DescTools_n_pow_sum", PACKAGE = "DescTools", x, meanx)
+      # proceed with the parameteric stuff
+      meanx <- mean.default(x)      # somewhat faster than mean
 
-    # this is method 3 in the usual functions Skew and Kurt
-    skewx <- ((1/n * psum$sum3) /  (psum$sum2 / n)^1.5) * ((n - 1)/n)^(3/2)
-    kurtx <- ((((1/n * psum$sum4) /  (psum$sum2 / n)^2) - 3)  + 3) * (1 - 1/n)^2 - 3
+      # we send the SORTED vector WITHOUT NAs to the C++ function to calc the power sum(s)
+      psum <- .Call("DescTools_n_pow_sum", PACKAGE = "DescTools", x, meanx)
 
-    sdx <- (psum$sum2 / (n-1))^0.5
+      # this is method 3 in the usual functions Skew and Kurt
+      skewx <- ((1/n * psum$sum3) /  (psum$sum2 / n)^1.5) * ((n - 1)/n)^(3/2)
+      kurtx <- ((((1/n * psum$sum4) /  (psum$sum2 / n)^2) - 3)  + 3) * (1 - 1/n)^2 - 3
 
-# changed for 0.99.12: leave out the test (as Werner Stahel advised against)
-#     if (n %[]% c(5, 5000)) {
-#       tres <- tryCatch(shapiro.test(x), error = function(e) { e })
-#     } else {
-#       tres <- tryCatch(AndersonDarlingTest(x, null = "pnorm"), error = function(e) { e })
-#     }
+      sdx <- (psum$sum2 / (n-1))^0.5
 
-#     vals <- prettyNum(c(psum$small_val, psum$large_val), big.mark = "'")
-#     frq <- c(psum$small_freq, psum$large_freq)
-#     frqtxt <- paste(" (", Format(frq, fmt=.fmt_abs()), ")", sep = "")
-#     frqtxt[frq < 2] <- ""
-#     txt <- StrTrim(paste(vals, frqtxt, sep = ""))
-#     lowtxt <- paste(head(txt, min(length(psum$small_val), nlow)), collapse = ", ")
-#     hightxt <- paste(rev(tail(txt, min(length(psum$large_val), nhigh))), collapse = ", ")
+  # changed for 0.99.12: leave out the test (as Werner Stahel advised against)
+  #     if (n %[]% c(5, 5000)) {
+  #       tres <- tryCatch(shapiro.test(x), error = function(e) { e })
+  #     } else {
+  #       tres <- tryCatch(AndersonDarlingTest(x, null = "pnorm"), error = function(e) { e })
+  #     }
+
+  #     vals <- prettyNum(c(psum$small_val, psum$large_val), big.mark = "'")
+  #     frq <- c(psum$small_freq, psum$large_freq)
+  #     frqtxt <- paste(" (", Format(frq, fmt=.fmt_abs()), ")", sep = "")
+  #     frqtxt[frq < 2] <- ""
+  #     txt <- StrTrim(paste(vals, frqtxt, sep = ""))
+  #     lowtxt <- paste(head(txt, min(length(psum$small_val), nlow)), collapse = ", ")
+  #     hightxt <- paste(rev(tail(txt, min(length(psum$large_val), nhigh))), collapse = ", ")
+
+      # put together the results
+      res <- list(length=ntot, n=n, NAs=NAs, unique=psum$unique, "0s"=psum$zero,
+                  mean=meanx, meanSE = sdx/sqrt(n),
+                  quant=qs,
+                  range=unname(diff(qs[c(1,9)])),
+                  sd=sdx, vcoef=sdx/meanx,
+                  mad=mad(x, center = qs[5]), IQR=unname(diff(qs[c(4,6)])),
+                  skew=skewx, kurt=kurtx,
+                  small=data.frame(val=psum$small_val, freq=psum$small_freq),
+                  large=data.frame(val=psum$large_val, freq=psum$large_freq)
+      )
 
 
+    } else {
 
-    # put together the results
-    res <- list(length=ntot, n=n, NAs=NAs, unique=psum$unique, "0s"=psum$zero,
-                mean=meanx, meanSE = sdx/sqrt(n),
-                quant=qs,
-                range=unname(diff(qs[c(1,9)])),
-                sd=sdx, vcoef=sdx/meanx,
-                mad=mad(x, center = qs[5]), IQR=unname(diff(qs[c(4,6)])),
-                skew=skewx, kurt=kurtx,
-                small=data.frame(val=psum$small_val, freq=psum$small_freq),
-                large=data.frame(val=psum$large_val, freq=psum$large_freq)
-#                 lowtxt=lowtxt, hightxt=hightxt
-    )
+      # put together the results
+      res <- list(length=ntot, n=n, NAs=NAs, unique=0, "0s"=0,
+                  mean=NA, meanSE = NA,
+                  quant=setNames(rep(NA,9), c("min", ".05", ".10", ".25", "median", ".75", ".90", ".95", "max")),
+                  range=NA,
+                  sd=NA, vcoef=NA,
+                  mad=NA, IQR=NA,
+                  skew=NA, kurt=NA,
+                  small=data.frame(val=NA, freq=NA),
+                  large=data.frame(val=NA, freq=NA)
+      )
+
+    }
+
+    return(res)
 
   }
 
@@ -124,7 +142,7 @@ Desc.numeric <- function (x, main = NULL, highlow = TRUE, digits = NULL,
 
   lst <- list(l1 = unlist(zz[c(1:7)]), l2=zz[["quant"]][-c(1,9)], l3=unlist(zz[c(9:15)]))
 
-  width <- max(c(unlist(lapply(lst, nchar)), unlist(lapply(lapply(lst, names), nchar))))
+  width <- max(c(unlist(lapply(lst, nchar)), unlist(lapply(lapply(lst, names), nchar))), na.rm=TRUE)
   if (z$unique == z$n)
     lst$l1["unique"] <- "= n"
 
@@ -143,22 +161,45 @@ Desc.numeric <- function (x, main = NULL, highlow = TRUE, digits = NULL,
   # if(defdigits) vals <- gsub("\\.$", "\\.0", gsub("0+$", "", vals))
   if(defdigits) vals <- gsub("\\.0+$", "\\.0", gsub("^(\\d+\\.\\d*?[1-9])0+$", "\\1", vals, perl = TRUE))
 
-  frq <- c(z$small$freq, z$large$freq)
-  frqtxt <- paste(" (", Format(frq, fmt=.fmt_abs()), ")", sep = "")
-  frqtxt[frq < 2] <- ""
-  txt <- StrTrim(paste(vals, frqtxt, sep = ""))
-  z$lowtxt <- paste(head(txt, min(length(z$small$val), nlow)), collapse = ", ")
-  z$hightxt <- paste(rev(tail(txt, min(length(z$large$val), nhigh))), collapse = ", ")
+  if(z$unique < 10 && z$n > 0) {
+    # we table the frequencies if there's less than 10 unique values
+
+    ff <- Sort(merge(data.frame(z$small$val, z$small$freq), data.frame(z$large$val, z$large$freq),
+                     by.x = 1, by.y = 1, all.x=TRUE, all.y=TRUE))
+    ff$n <- do.call(Coalesce, ff[, c(2,3)])
+    tab <- as.table(ff$n)
+    names(tab) <- signif(ff[,1])
+    f <- Freq(tab)
+
+    z$lowtxt <- NA
+    z$hightxt <- NA
+    z$freq <- f
+
+  } else{
+    frq <- c(z$small$freq, z$large$freq)
+    frqtxt <- paste(" (", Format(frq, fmt=.fmt_abs()), ")", sep = "")
+    frqtxt[frq < 2] <- ""
+    txt <- StrTrim(paste(vals, frqtxt, sep = ""))
+    z$lowtxt <- paste(head(txt, min(length(z$small$val), nlow)), collapse = ", ")
+    z$hightxt <- paste(rev(tail(txt, min(length(z$large$val), nhigh))), collapse = ", ")
+    z$freq <- NA
+  }
 
   if (highlow) {
-    cat(paste("lowest : ", z$lowtxt, "\n", "highest: ", z$hightxt, "\n\n", sep = ""))
+    if(identical(z$freq, NA)){
+      cat(paste("lowest : ", z$lowtxt, "\n", "highest: ", z$hightxt, "\n\n", sep = ""))
+    } else {
+      cat("\n")
+      print(f)
+      cat("\n")
+    }
   }
 
   if (plotit)
     if (z$n > 0)
       PlotDesc.numeric(x, main = main)
     else
-      cat(gettextf("Nothing to plot in %s", deparse(substitute(x))))
+      cat(gettextf("Nothing to plot in %s\n\n", deparse(substitute(x))))
 
   invisible(z)
 
@@ -175,14 +216,22 @@ Desc.factor <- function (x, main = NULL, ord = c("desc", "asc", "name", "level")
 
     nan <- length(x)
     x <- na.omit(x)
+    n <- length(x)
 
-    f <- Freq(x, ord=ord)
-
-    res <- list(length=nan, n=tail(f$cumfreq, 1),
-                NAs = nan - tail(f$cumfreq, 1), levels=nlevels(x),
-                unique=sum(f$freq>0), dupes=any(f$freq>1),
-                frq = f
-    )
+    if(n > 0){
+      f <- Freq(x, ord=ord)
+      res <- list(length=nan, n=n,
+                  NAs = nan - n, levels=nlevels(x),
+                  unique=sum(f$freq>0), dupes=any(f$freq>1),
+                  frq = f
+      )
+    } else {
+      res <- list(length=nan, n=n,
+                  NAs = nan - n, levels=0,
+                  unique=0, dupes=0,
+                  frq = NA
+      )
+    }
 
     return(res)
   }
@@ -219,26 +268,32 @@ Desc.factor <- function (x, main = NULL, ord = c("desc", "asc", "name", "level")
   cat(paste(.txtline(lfmt, width = width, ind = "  ", space = " "),
             collapse = "\n"))
 
-  if (is.na(maxrows)) {
-    maxrows <- nrow(z$frq)
+  if(z$n > 0){   # print that only if there's real data
+    if (is.na(maxrows)) {
+      maxrows <- nrow(z$frq)
+    }
+    if (maxrows < 1) {
+      maxrows <- sum(z$frq[, 5] < maxrows) + 1
+    }
+    lfmt$frq <- z$frq[1:min(nrow(z$frq), maxrows), ]
+    txt.frq <- .CaptOut(lfmt$frq)
+  #   txt1.frq <- paste(substr(txt.frq, 0, regexpr("level", txt.frq[1]) + 5),
+  #                         x = substr(txt.frq, regexpr("level", txt.frq[1]) + 5 + 1,
+  #                                    nchar(txt.frq[1])),
+  #                    sep = "")
+    cat("\n")
+    cat(txt.frq, sep = "\n")
+    if (maxrows < z$levels)
+      cat("... etc.\n [list output truncated]\n\n")
+    else cat("\n")
   }
-  if (maxrows < 1) {
-    maxrows <- sum(z$frq[, 5] < maxrows) + 1
-  }
-  lfmt$frq <- z$frq[1:min(nrow(z$frq), maxrows), ]
-  txt.frq <- .CaptOut(lfmt$frq)
-#   txt1.frq <- paste(substr(txt.frq, 0, regexpr("level", txt.frq[1]) + 5),
-#                         x = substr(txt.frq, regexpr("level", txt.frq[1]) + 5 + 1,
-#                                    nchar(txt.frq[1])),
-#                    sep = "")
-  cat("\n")
-  cat(txt.frq, sep = "\n")
-  if (maxrows < z$levels)
-    cat("... etc.\n [list output truncated]\n\n")
-  else cat("\n")
 
   if (plotit)
-    PlotDesc.factor(x, main = main, maxrows = maxrows, ord = ord, ecdf=TRUE, ...)
+    if (z$n > 0)
+      PlotDesc.factor(x, main = main, maxrows = maxrows, ord = ord, ecdf=TRUE, ...)
+  else
+    cat(gettextf("\nNothing to plot in %s\n\n", deparse(substitute(x))))
+
 
   invisible(z)
 
@@ -315,22 +370,29 @@ Desc.logical <- function (x, main = NULL, digits = NULL, conf.level = 0.95,
   cat(paste(.txtline(lfmt, width = width, ind = "  ", space = " "),
             collapse = "\n"), "\n")
 
-  out <- cbind(
-           freq=Format(z$afrq, fmt = .fmt_abs()),
-                Format(z$rfrq, fmt = .fmt_per()))
+  if(z$n > 0) {
+    out <- cbind(
+             freq=Format(z$afrq, fmt = .fmt_abs()),
+                  Format(z$rfrq, fmt = .fmt_per()))
 
-  rownames(out) <- rownames(z$afrq)
-  colnames(out) <- c("freq", "perc",
-                     gettextf(c("lci%s", "uci%s"),
-                              Format(conf.level, digits=2, leading="drop")))
+    rownames(out) <- rownames(z$afrq)
+    colnames(out) <- c("freq", "perc",
+                       gettextf(c("lci%s", "uci%s"),
+                                Format(conf.level, digits=2, leading="drop")))
+
+    txt <- capture.output(print(out, quote=FALSE, right=TRUE, print.gap=2))
+    cat(paste(txt[1], Coalesce(getOption("footnote1"), "'"),
+              sep = ""), txt[-1], sep = "\n")
+    cat(gettextf("\n%s %s%s-CI Wilson\n\n", Coalesce(getOption("footnote1"),
+                                                     "'"), conf.level * 100, "%"))
+  }
 
-  txt <- capture.output(print(out, quote=FALSE, right=TRUE, print.gap=2))
-  cat(paste(txt[1], Coalesce(getOption("footnote1"), "'"),
-            sep = ""), txt[-1], sep = "\n")
-  cat(gettextf("\n%s %s%s-CI Wilson\n\n", Coalesce(getOption("footnote1"),
-                                                   "'"), conf.level * 100, "%"))
   if (plotit)
-    PlotDesc.logical(x, main = main)
+    if (z$n > 0)
+      PlotDesc.logical(x, main = main)
+  else
+    cat(gettextf("Nothing to plot in %s\n\n", deparse(substitute(x))))
+
 
   invisible(z)
 
@@ -381,8 +443,14 @@ Desc.integer <- function (x, main = NULL, maxrows = 12, freq = NULL, digits = NU
       }
     }
   }
+
+
   if (plotit)
-    PlotDesc.integer(x, main = main, maxrows = maxrows)
+    if (lres$n > 0)
+      PlotDesc.integer(x, main = main, maxrows = maxrows)
+  else
+    cat(gettextf("Nothing to plot in %s\n\n", deparse(substitute(x))))
+
 
   invisible(lres)
 }
